/* 
 Authent 1.4
 Ralph Pugmire
 Netwin Ltd.
 August 1997

 An example authentication program for use with DPOP and DMAIL
 This program is spawned as a sub process by DPOP
 DPOP sends commands to authent stdin via a pipe
 and receives output from authent stdout via a pipe
 Authent just reads commands with gets
 and writes replies with printf  
 BUT fflush must be used after each printf

 Commands:
	lookup user@domain
	lookup user
	check user@domain password fromip
	check user password fromip
	exit

  Replies:
	-ERR error message
	+OK user path uid forward
 
 Notes 
	DPOP may start several instances of this program to improve throughput 
	Returned path should include drop file name. eg /spool/mail/fred 
	Use ffllush after every printf
	For use with DPOP the returned field 'forward' is not used and may be left blank
	If virtual domains are not being used the '@domain' passed with username may be ignored
	The @domain will only be passed if   authent_domain true is put in dpop.conf

 Modification History 
	RHP 3/9/97	Accept new format lookup fred@a.b.c.d and extra return field 'forward'
*/

#include <stdio.h>
#include <string.h>

#ifdef WIN32
#include <windows.h>
#include <io.h>
#define sleep(a) Sleep(a * 1000)
#endif

#ifdef unix
#endif

#ifndef false
#define false 0
#define true !false
#endif


/* Prototypes */
int check(char *user, char *domain, char *pass,char *from, char *path, char *uid, char *forward);
int lookup(char *user, char *domain, char *path, char *uid, char *forward);

void main(void){
	char bf[100];
	char *cmd, *user, *domain, *pass, *from, *p;
	char path[100];
	char uid[100];
	char forward[100];
	const char SEP[]=" \t\r\n";
	int goodlabtime;

	/* Note any unsolicited prints like this would cause problems */
	 /*printf("Authent version 1.1\n"); /* this print may cause probs */
	 /*fflush(stdout);*/

	/* for testing we insert a large delay at startup and
	   a smaller delay on each transaction.
	   These sleep() calls should obviously
	   be removed in any real version of authent
	*/
	/* sleep(10); */
	while(1){
		gets(bf);
		 /* sleep(3); /* For test we make it slow to check non blocking */
		 /* Example constraint: Is it a good time for labip's to read mail */
		goodlabtime = false;
		cmd = strtok(bf,SEP); if (cmd==NULL) cmd="";
		if (strcmp(cmd,"check")==0){
			/* CHECK A USER/PASSWORD/FROMIP */
			user = strtok(NULL,SEP);
			if ((p=strchr(user,'@'))!=NULL){
				*p=0;
				domain = ++p;
			}else
				domain = "";
			pass = strtok(NULL,SEP);
			from = strtok(NULL,SEP);
			if (from==NULL) from = "";
			if (!check(user,domain,pass,from,path,uid,forward))
				printf("-ERR %s invalid username/password\n", user);
			else if (strstr(from,"labip")!=NULL && !goodlabtime)
				printf("-ERR %s cant read mail from lab now\n", user);
			else 
				printf("+OK %s %s %s %s\n", user, path, uid, forward);
		}else if (strcmp(cmd,"lookup")==0){
			/* LOOKUP a USER */
			user = strtok(NULL,SEP);
			if ((p=strchr(user,'@'))!=NULL){
				*p=0;
				domain = ++p;
			}else
				domain = "";
			if (lookup(user,domain,path,uid,forward))
				printf("+OK %s %s %s %s\n", user, path, uid, forward);
			else 
				printf("-ERR %s invalid username\n", user);
		}else if (strcmp(cmd,"exit")==0){
			return;
		}else{
			/* Invalid command */
			printf("-ERR Invalid command: %s\n", cmd);
		}
		/* printf("extra stuff\n");/*  to check dpop clears buffer after and before response */
		fflush(stdout);
	}
}

/* TRIVIAL SAMPLE DATABASE  only user is ralph password ralph*/
int check(char *user, char *domain, char *pass, char *from, char *path, char *uid, char *forward){
	/* we just ignore domain */
	/* check user/ password combo */
	if (user==NULL) return false;
	if (pass==NULL) return false;
	if (strcmp(user,"ralph")==0){
		if (strcmp(pass,"ralph")==0)
		return lookup(user,domain,path,uid,forward);
	}else if (strcmp(user,"sleep")==0){
		sleep(120);
		if (strcmp(pass,"ralph")==0)
		return lookup("ralph",domain,path,uid,forward);
	}else if (strcmp(user,"mrhash")==0){
		if (strcmp(pass,"hash")==0)
		return lookup(user,domain,path,uid,forward);
	}else if (strcmp(user,"die")==0){
		exit(0);
	}else if (strcmp(user,"nouid")==0){
		if (strcmp(pass,"nouid")==0)
		return lookup(user,domain,path,uid,forward);
	}else if (strncmp(user,"test",4)==0){
		if (strcmp(pass,"test")==0)
		return lookup(user,domain,path,uid,forward);
	}else if (strncmp(user,"die",3)==0){
		exit(1);
	}else if (strncmp(user,"dpop",9)==0){
		if (strcmp(pass,"test")==0)
		return lookup(user,domain,path,uid,forward);
	}
	return false;
}

#ifdef unix
int lookup(char *user, char *domain, char *path, char *uid, char *forward){
	/* we just ignore domain*/
	if (user==NULL) return false;
	if (strcmp(user,"ralph")==0){
		strcpy(path,"/var/spool/mail/ralph");
		sprintf(uid,"%s","ralph");
		sprintf(forward, "domain=%s",domain); /* return domain as forward as a test */
		return true;
	}else if (strcmp(user,"mrhash")==0){
		strcpy(path,"/usr/local/dpop/dropdir#");
		sprintf(uid,"%d",23);
		return true;
	}else if (strcmp(user,"nouid")==0){
		strcpy(path,"/usr/local/dpop/dropdir#");
		sprintf(uid,"");
		return true;
	}else if (strncmp(user,"test",4)==0){
		sprintf(path,"/var/spool/mail/%s",user);
		sprintf(uid,"%s%c","test",user[strlen(user)-1]);
		return true;
	}else if (strncmp(user,"dpop.test",9)==0){
		sprintf(path,"/usr/spool/netmail/00/dpopt%s",&user[10]);
		sprintf(uid,"dpopt%s",&user[10]);
		return true;
	}else
		return false;
}
#endif
#ifdef WIN32
int lookup(char *user, char *domain, char *path, char *uid, char *forward){
	/* we just ignore domain */
	if (user==NULL) return false;
	if (strcmp(user,"ralph")==0){
		strcpy(path,"config");
		sprintf(uid,"%d",1234);
		sprintf(forward, "domain=%s",domain); /* return domain as forward as a test */
		return true;
	}else if (strcmp(user,"mrhash")==0){
		strcpy(path,"c:\\dpop\\dropdir\\mrhash");
		sprintf(uid,"%d",23);
		return true;
	}else if (strncmp(user,"test",4)==0){
		strcpy(path,"c:\\dpop\\testdropdir\\");
		strcat(path,user);
		sprintf(uid,"%s",&user[4]);
		return true;
	}else if (strncmp(user,"dpop.test",9)==0){
		sprintf(path,"/usr/spool/netmail/00/dpopt%s", &user[10]);
		sprintf(uid,"dpopt%s",&user[10]);
		return true;
	}else
		return false;
}
#endif